//
//  BouncingWallsScene.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 06.08.17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import UIKit
import SpriteKit
import CoreMotion

struct BoneBody {
    var node:SKSpriteNode!
    var body:SKPhysicsBody!
    var position:CGPoint!
    var rotation:CGFloat!
}

extension FloatingPoint {
    var degreesToRadians: Self { return self * .pi / 180 }
    var radiansToDegrees: Self { return self * 180 / .pi }
}

class BouncingWallsScene: SKScene {
    
    //  Scene representations of the UI
    var hasLaidOut = false
    var statusBarBlocker: SKShapeNode!
    var topText: SKShapeNode!
    var bottomButton = SKSpriteNode(imageNamed: "next_arrow_white")
    var resetButton: SKShapeNode!
    
    // Accelerometer support
    var motionManager = CMMotionManager()
    var isLandscape = false
    
    // Touch handling
    var selectedNode: SKSpriteNode!
    var touchPoint: CGPoint = CGPoint()
    var touching: Bool = false
    
    // Skeleton
    var skeletonContainer: SKNode!
    var parts:Array = [BoneBody]()
    
    
    // MARK: - Setup
    override func didMove(to view: SKView) {
        
        super.didMove(to: view)
        
        backgroundColor = SKColor.black
        
        physicsWorld.gravity = CGVector(dx: 0.0, dy: 0.0)
        
        skeletonContainer = childNode(withName: "skeletonContainer")
        createBodyParts()
        
        
        
        statusBarBlocker = SKShapeNode(rectOf: CGSize(width: 0, height: 0))
        statusBarBlocker.fillColor = SKColor.clear
        statusBarBlocker.lineWidth = 0
        addChild(statusBarBlocker)

        let bTexture = SKTexture(imageNamed: "next_arrow_white")
        bottomButton.physicsBody = SKPhysicsBody(texture: bTexture,
                                                 alphaThreshold: 0.5,
                                                 size: CGSize(width: bottomButton.size.width,
                                                              height: bottomButton.size.height))
        bottomButton.physicsBody?.pinned = true
        bottomButton.physicsBody?.allowsRotation = false
        bottomButton.alpha = 0
        addChild(bottomButton)
        
        resetButton = SKShapeNode(circleOfRadius: 17.0)
        resetButton.fillColor = SKColor.clear
        resetButton.lineWidth = 0
        resetButton.physicsBody = SKPhysicsBody(circleOfRadius: 17.0)
        resetButton.physicsBody?.pinned = true
        resetButton.physicsBody?.allowsRotation = false
        addChild(resetButton)
        
        // Control world gravity with acceleratometer
        if motionManager.isDeviceMotionAvailable == true {
            motionManager.startDeviceMotionUpdates(to:OperationQueue.current!, withHandler:{
                data, error in
                let gravity:CMAcceleration = data!.gravity
                
                let dx:CGFloat = CGFloat(gravity.x / 4)
                let dy:CGFloat = CGFloat(gravity.y / 4)
                
                if UIDevice.current.orientation == UIDeviceOrientation.landscapeLeft {
                    self.physicsWorld.gravity = CGVector(dx: dy, dy: dx)
                } else if UIDevice.current.orientation == UIDeviceOrientation.landscapeRight {
                    self.physicsWorld.gravity = CGVector(dx: dy, dy: -dx)
                } else if UIDevice.current.orientation == UIDeviceOrientation.portrait {
                    self.physicsWorld.gravity = CGVector(dx: dx, dy: dy)
                } else if UIDevice.current.orientation == UIDeviceOrientation.portraitUpsideDown {
                    self.physicsWorld.gravity = CGVector(dx: dx, dy: -dy)
                }
            })
        }
        layOutBlockerBodies()
    }
    
    
    override func didChangeSize(_ oldSize: CGSize) {
        
        if hasLaidOut == false { return }
        //print("didChangeSize")
        
        layOutBlockerBodies()
    }
    
    func layOutBlockerBodies() {
        
        let minX:CGFloat = -size.width/2
        let minY:CGFloat = -size.height/2
        let width:CGFloat = size.width
        let height:CGFloat = size.height
        let rect:CGRect = CGRect(x:minX, y:minY, width:width, height:height)
        
        // Set up the edge border
        let borderBody = SKPhysicsBody(edgeLoopFrom: rect)
        borderBody.friction = 0
        self.physicsBody = borderBody
        
        // Status bar blocker
        let barHeight:CGFloat = 20.0
        statusBarBlocker.position = CGPoint(x:0, y:height/2 - barHeight/2)
        statusBarBlocker.physicsBody = SKPhysicsBody(rectangleOf: CGSize(width:width, height:barHeight))
        statusBarBlocker.physicsBody?.pinned = true
        statusBarBlocker.physicsBody?.allowsRotation = false
        
        // Bottom button
        bottomButton.position = CGPoint(x:0, y:rect.minY + 50.0)
        
        // Reset button
        resetButton.position = CGPoint(x:rect.minX + 46, y:rect.minY + 47)
        
        hasLaidOut = true
    }
    

    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        for touch: AnyObject in touches {
            let location = touch.location(in: self)
            let touchedNode = atPoint(location)
            if touchedNode is SKSpriteNode {
                selectedNode = touchedNode as! SKSpriteNode
                touching = true
            }
        }
    }
    
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        for touch: AnyObject in touches {
            let location = touch.location(in: self)
            touchPoint = location
        }
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        touching = false
    }
    
    override func touchesCancelled(_ touches: Set<UITouch>, with event: UIEvent?) {
        touching = false
    }
    
    override func update(_ currentTime: TimeInterval) {
        if touching {
            //let dt:CGFloat = 1.0/60.0
            let distance = CGVector(dx: touchPoint.x-selectedNode.position.x, dy: touchPoint.y-selectedNode.position.y)
            //let velocity = CGVector(dx: distance.dx/dt, dy: distance.dy/dt)
            let velocity2 = CGVector(dx: distance.dx*2, dy: distance.dy*2)
            //selectedNode.physicsBody!.velocity = velocity
            //selectedNode.physicsBody?.applyForce(distance)
            selectedNode.physicsBody?.applyForce(velocity2)
        }
    }
    
    func addPart(_ name:String) {
        var boneBody = BoneBody()
        let n = skeletonContainer.childNode(withName: name) as! SKSpriteNode
        boneBody.node = n
        boneBody.body = n.physicsBody
        boneBody.position = n.position
        boneBody.rotation = n.zRotation
        parts.append(boneBody)
    }
    
    func createBodyParts() {
        addPart("jaw")
        addPart("skull")
        addPart("spine")
        addPart("hips")
        addPart("toparm_left")
        addPart("toparm_right")
        addPart("forearm_left")
        addPart("forearm_right")
        addPart("femur_right")
        addPart("femur_left")
        addPart("shin_right")
        addPart("shin_left")
        addPart("rib_r_5")
        addPart("rib_l_5")
        addPart("rib_r_4")
        addPart("rib_l_4")
        addPart("rib_r_3")
        addPart("rib_l_3")
        addPart("rib_r_2")
        addPart("rib_l_2")
        addPart("rib_r_1")
        addPart("rib_l_1")
        addPart("finger_r_5")
        addPart("finger_r_4")
        addPart("finger_r_3")
        addPart("finger_r_2")
        addPart("finger_r_1")
        addPart("finger_l_5")
        addPart("finger_l_4")
        addPart("finger_l_3")
        addPart("finger_l_2")
        addPart("finger_l_1")
        addPart("foot_left")
        addPart("foot_right")
        
        if UIDevice.current.userInterfaceIdiom == .pad {
            skeletonContainer.setScale(1.5)
        }
    }
    
    public func redoBody() {
        for b in parts {
            let move = SKAction.move(to: b.position, duration: 1.2)
            move.timingMode = SKActionTimingMode.easeInEaseOut
            let rotate = SKAction.rotate(toAngle: b.rotation, duration: 1.2)
            rotate.timingMode = SKActionTimingMode.easeInEaseOut
            let group = SKAction.group([move, rotate])
            b.node.physicsBody = nil
            b.node.run(group) {
                b.node.physicsBody = b.body
            }
        }
    }

}
