//
//  FallingBonesScene.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 10.08.17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import UIKit
import SpriteKit
import CoreMotion

extension Array {
    /// Returns an array containing this sequence shuffled
    var shuffled: Array {
        var elements = self
        return elements.shuffle()
    }
    /// Shuffles this sequence in place
    @discardableResult
    mutating func shuffle() -> Array {
        indices.dropLast().forEach { a in
            guard case let b = Int(arc4random_uniform(UInt32(count - a))) + a, b != a else { return }
            swapAt(a, b)
        }
        return self
    }
    var chooseOne: Element { return self[Int(arc4random_uniform(UInt32(count)))] }
    func choose(_ n: Int) -> Array { return Array(shuffled.prefix(n)) }
}

class FallingBonesScene: SKScene {
    
    // Accelerometer support
    var motionManager = CMMotionManager()
    
    // Bones bones bones
    var containerNode = SKNode()
    var b1:SKSpriteNode!
    var currentBones = [SKSpriteNode]()
    var isAscending = false
    var bottomButton = SKSpriteNode(imageNamed: "next_arrow_white")
    var selectionBlocker = SKShapeNode(rectOf: CGSize(width: 56.0, height: 56.0))
    
    // Touch handling
    var selectedNode: SKSpriteNode!
    var touchPoint: CGPoint = CGPoint()
    var touching: Bool = false
    var numOfNodes: Int = 0
    var currentAge: Int = 0
    
    // MARK: - Setup
    override func didMove(to view: SKView) {
    
        b1 = childNode(withName: "b1") as? SKSpriteNode
        b1.removeFromParent()
        
        addChild(containerNode)
        
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        updateBones(number: appDelegate.startAge)
        
        let bTexture = SKTexture(imageNamed: "next_arrow_white")
        bottomButton.physicsBody = SKPhysicsBody(texture: bTexture,
                                                 alphaThreshold: 0.5,
                                                 size: CGSize(width: bottomButton.size.width,
                                                              height: bottomButton.size.height))
        bottomButton.physicsBody?.pinned = true
        bottomButton.physicsBody?.allowsRotation = false
        bottomButton.alpha = 0
        addChild(bottomButton)
        
        selectionBlocker.position = CGPoint(x:0, y: 0)
        selectionBlocker.fillColor = SKColor.clear
        selectionBlocker.lineWidth = 0
        selectionBlocker.physicsBody = SKPhysicsBody(rectangleOf: CGSize(width:64.0, height:64.0))
        selectionBlocker.physicsBody?.pinned = true
        selectionBlocker.physicsBody?.allowsRotation = false
        addChild(selectionBlocker)
        
        // Control world gravity with acceleratometer
        if motionManager.isDeviceMotionAvailable == true {
            motionManager.startDeviceMotionUpdates(to:OperationQueue.current!, withHandler:{
                data, error in
                let gravity:CMAcceleration = data!.gravity
                
                let dx:CGFloat = CGFloat(gravity.x / 4)
                let dy:CGFloat = CGFloat(gravity.y / 4)
                
                if UIDevice.current.orientation == UIDeviceOrientation.landscapeLeft {
                    self.physicsWorld.gravity = CGVector(dx: dy, dy: dx)
                } else if UIDevice.current.orientation == UIDeviceOrientation.landscapeRight {
                    self.physicsWorld.gravity = CGVector(dx: dy, dy: -dx)
                } else if UIDevice.current.orientation == UIDeviceOrientation.portrait {
                    self.physicsWorld.gravity = CGVector(dx: dx, dy: dy)
                } else if UIDevice.current.orientation == UIDeviceOrientation.portraitUpsideDown {
                    self.physicsWorld.gravity = CGVector(dx: dx, dy: -dy)
                }
            })
        }
        
    }
    
    override func didChangeSize(_ oldSize: CGSize) {
        
        let minX:CGFloat = -size.width/2
        let minY:CGFloat = -size.height/2
        let width:CGFloat = size.width
        let height:CGFloat = size.height
        let rect:CGRect = CGRect(x:minX, y:minY, width:width, height:height)
        
        // Bottom button
        bottomButton.position = CGPoint(x:0, y:rect.minY + 50.0)
    }
    
    override func update(_ currentTime: TimeInterval) {
        super.update(currentTime)
        
        containerNode.enumerateChildNodes(withName: "//*", using:
            { (node, stop) -> Void in
                if (!self.intersects(node)) {
                    self.updateOutOfScreenPosition(node:node)
                }
            })
        
//        if touching {
//            let distance = CGVector(dx: touchPoint.x-selectedNode.position.x, dy: touchPoint.y-selectedNode.position.y)
//            let velocity2 = CGVector(dx: distance.dx*2, dy: distance.dy*2)
//            selectedNode.physicsBody?.applyForce(velocity2)
//        }
    }
    
    func updateOutOfScreenPosition(node:SKNode) {
        var newPosition     = CGPoint(x:node.position.x, y:node.position.y)
        let offset:CGFloat  = 50
        let startY:CGFloat  = frame.maxY + offset
        let endY:CGFloat    = frame.minY - offset
        let bottomY:CGFloat = frame.minY - node.frame.height/2 - (offset*2)
        let topY:CGFloat    = frame.maxY + node.frame.height/2 + (offset*2)
        let startX:CGFloat  = frame.minX - offset
        let endX:CGFloat    = frame.maxX + offset
        let leftX:CGFloat   = frame.minX - node.frame.width/2 - (offset*2)
        let rightX:CGFloat  = frame.maxX + node.frame.width/2 + (offset*2)
        let nodeY:CGFloat   = node.position.y
        let nodeX:CGFloat   = node.position.x
        if (nodeY < bottomY) {
            newPosition.y = startY
        }
        else if (nodeY > topY) {
            newPosition.y = endY
        }
        if (nodeX > rightX) {
            newPosition.x = startX
        }
        else if (nodeX < leftX) {
            newPosition.x = endX
        }
        //print(nodeY, bottomY, startY)
        node.position = newPosition
    }
    
    func updateBones(number:Int) {
        if currentAge < number {
            isAscending = true
        } else {
            isAscending = false
        }
        currentAge = number
        numOfNodes = currentBones.count
        if numOfNodes == number { return }
        if numOfNodes < number {
            createBones(number: number - numOfNodes)
        } else if numOfNodes > number {
            deleteBones(number: numOfNodes - number)
        }
        applyImpulses()
        //numOfNodes = children.count
        //print("updated numOfNodes", numOfNodes)
    }
    
    func createBones(number:Int) {
        for i in 0 ... number-1 {
            let s:SKSpriteNode = b1.copy() as! SKSpriteNode
            s.name = i.description
            let x = frame.minX + (CGFloat(drand48()) * frame.width)
            let y = frame.minY + (CGFloat(drand48()) * frame.height)
            s.position = CGPoint(x:x, y:y)
            s.physicsBody?.angularVelocity = CGFloat(arc4random_uniform(10)) * CGFloat(drand48())
            containerNode.addChild(s)
            
            s.scale(to: CGSize(width: 0, height: 0))
            s.alpha = 0
            let scaleUp = SKAction.scale(to: 1.0, duration: 0.5)
            scaleUp.timingMode = .easeOut
            let fadeIn = SKAction.fadeIn(withDuration: 0.5)
            fadeIn.timingMode = .easeOut
            s.run(scaleUp)
            s.run(fadeIn)
        
            currentBones.insert(s, at: i)
        }
    }
    
    func deleteBones(number:Int) {
        for index in stride(from: currentBones.count-1, to: currentBones.count-1 - number, by: -1) {
            let node = containerNode.children[index] as SKNode
            let fadeOut:SKAction = SKAction.fadeOut(withDuration: 0.5)
            fadeOut.timingMode = .easeOut
            let scaleDown = SKAction.scale(to: 0.0, duration: 0.5)
            scaleDown.timingMode = .easeOut
            node.run(scaleDown)
            node.run(fadeOut) {
                node.removeFromParent()
            }
            currentBones.remove(at: index)
        }
    }
    
    func applyImpulses() {
        var direction:CGFloat = 1
        if isAscending { direction = -1 }
        let moveAmount:CGFloat = 50.0 * direction
        for node in currentBones {
            let distance = CGVector(dx: 0, dy: node.position.y + moveAmount)
            let velocity = CGVector(dx: distance.dx*2, dy: distance.dy*2)
            node.physicsBody?.applyForce(velocity)
        }
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        for touch: AnyObject in touches {
            let location = touch.location(in: self)
            let touchedNode = atPoint(location)
            if touchedNode is SKSpriteNode {
                selectedNode = touchedNode as! SKSpriteNode
                touching = true
            }
        }
    }
    
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        for touch: AnyObject in touches {
            let location = touch.location(in: self)
            touchPoint = location
        }
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        touching = false
    }
    
    override func touchesCancelled(_ touches: Set<UITouch>, with event: UIEvent?) {
        touching = false
    }
    
    public func onPickerChange(value:Int) {
        //print("onPickerChange", value)
        updateBones(number:value)
    }
    
}
